/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001 Nokia Corporation
   Original Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* 
   $Id: btcore.h,v 1.34 2002/12/13 12:10:29 kds Exp $

   HCI access library

   Fixes:	Dmitry Kasatkin <dmitry.kasatkin@nokia.com>
*/

#ifndef	_BTCORE_H
#define	_BTCORE_H

#include <fcntl.h>
#include <net/if.h>
#include <unistd.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include <syslog.h>

#include <affix/hci_types.h>
#include <affix/bluetooth.h>

__BEGIN_DECLS

extern char		*affix_version;
extern unsigned long	btlog_mask;

extern char	btdev[IFNAMSIZ];
extern int	linkmode;
extern int	sdpmode;

extern char 		*hci_errlist[];
extern char 		*lmp_compid[];

struct affix_tupla {
	unsigned int	value;
	char		*match;
	char		*str;
};


extern struct affix_tupla debug_flags_map[];

extern struct affix_tupla codServiceClassMnemonic[];
extern struct affix_tupla codMajorClassMnemonic[];
extern struct affix_tupla codMinorComputerMnemonic[];
extern struct affix_tupla codMinorPhoneMnemonic[];
extern struct affix_tupla codMinorAudioMnemonic[];

extern struct affix_tupla pkt_type_map[];
extern struct affix_tupla sec_level_map[];
extern struct affix_tupla sec_mode_map[];
extern struct affix_tupla role_map[];
extern struct affix_tupla scan_map[];


#define NUM_LMPCOMPID	37



#define BTDEBUG_MODULE_CONTROL	0x01
#define BTDEBUG_MODULE_SDP	0x02
#define BTDEBUG_MODULE_OBEX	0x04
#define BTDEBUG_MODULE_HFP	0x08

#ifndef BTDEBUG_MODULE 
#define BTDEBUG_MODULE 0
#endif


void btlog_hexdump(const char *fname, const unsigned char *data, int len);
int become_daemon(int do_fork);

#ifdef CONFIG_AFFIX_DEBUG
#define BTDEBUG(fmt, args...)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						syslog(LOG_DEBUG, __FUNCTION__": " fmt "\n", ##args);\
				} while (0)

#define BTDUMP(data, len)	do { \
					if (btlog_mask & BTDEBUG_MODULE)\
						btlog_hexdump(__FUNCTION__, data, len);\
				} while (0)
#else
#define BTDEBUG(args...)
#define BTDUMP(data, len)
#endif

#ifdef __cplusplus
#define BTDBG(fmt, args...)		syslog(LOG_DEBUG, fmt "\n", ##args)
#define BTERROR(fmt, args...)		syslog(LOG_ERR, fmt "\n", ##args)
#else
#define BTDBG(fmt, args...)		syslog(LOG_DEBUG, __FUNCTION__ ": " fmt "\n", ##args)
#define BTERROR(fmt, args...)		syslog(LOG_ERR, __FUNCTION__ ": " fmt "\n", ##args)
#endif
#define BTWARN(fmt, args...)		syslog(LOG_WARNING, __FUNCTION__ ": " fmt "\n", ##args)
#define BTNOTICE(fmt, args...)		syslog(LOG_NOTICE, __FUNCTION__ ": " fmt "\n", ##args)
#define BTINFO(fmt, args...)		syslog(LOG_INFO, __FUNCTION__ ": " fmt "\n", ##args)


// General purpose functions
char *hcierror(int err);
int str2bda(BD_ADDR *p, char *str);
void _bda2str(char *str, BD_ADDR *p);
char *bda2str(BD_ADDR *bda);


/* general hci */

static inline int hci_set_mode(int fd, int mode)
{
	return btsys_ioctl(fd, BTIOC_SET_CTL, &mode);
}

static inline int hci_get_attr(char *name, struct hci_dev_attr *attr)
{
	int	fd, err;
	fd = btsys_socket(PF_AFFIX, SOCK_RAW, BTPROTO_HCI);
	if (fd < 0)
		return fd;
	strncpy(attr->name, name, IFNAMSIZ);
	attr->hd = 0;
	err = btsys_ioctl(fd, BTIOC_GET_ATTR, attr);
	close(fd);
	return err;
}

static inline int hci_get_attr_id(int hd, struct hci_dev_attr *attr)
{
	int	fd, err;
	fd = btsys_socket(PF_AFFIX, SOCK_RAW, BTPROTO_HCI);
	if (fd < 0)
		return fd;
	attr->hd = hd;
	err = btsys_ioctl(fd, BTIOC_GET_ATTR, attr);
	close(fd);
	return err;
}

static inline int hci_get_attr_fd(int fd, struct hci_dev_attr *attr)
{
	return btsys_ioctl(fd, BTIOC_GET_ATTR, attr);
}

static inline int hci_set_attr(int fd, struct hci_dev_attr *attr)
{
	return btsys_ioctl(fd, BTIOC_SET_ATTR, attr);
}

static inline int hci_get_flags(int fd, int *flags)
{
	return btsys_ioctl(fd, BTIOC_GET_FLAGS, flags);
}

static inline int hci_set_flags(int fd, int flags)
{
	return btsys_ioctl(fd, BTIOC_SET_FLAGS, &flags);
}

#if 0
static inline int hci_set_flags(int fd, int flags)
{
	int 	err, old_flags;
	err = btsys_ioctl(fd, BTIOC_GET_FLAGS, &old_flags);
	if (err)
		return err;
	old_flags |= flags;
	return btsys_ioctl(fd, BTIOC_SET_FLAGS, &old_flags);
}

static inline int hci_clr_flags(int fd, int flags)
{
	int 	err, old_flags;
	err = btsys_ioctl(fd, BTIOC_GET_FLAGS, &old_flags);
	if (err)
		return err;
	old_flags &= ~flags;
	return btsys_ioctl(fd, BTIOC_SET_FLAGS, &old_flags);
}
#endif

static inline int hci_get_pkttype(int fd, int *pkt_type)
{
	return btsys_ioctl(fd, BTIOC_GET_PKTTYPE, pkt_type);
}

static inline int hci_set_pkttype(int fd, int pkt_type)
{
	return btsys_ioctl(fd, BTIOC_SET_PKTTYPE, &pkt_type);
}

static inline int hci_set_secmode(int fd, int secmode)
{
	return btsys_ioctl(fd, BTIOC_SET_SECMODE, &secmode);
}

static inline int hci_set_role(int fd, int role)
{
	return btsys_ioctl(fd, BTIOC_SET_ROLE, &role);
}

static inline int hci_get_version(int fd, struct affix_version *ver)
{
	return btsys_ioctl(fd, BTIOC_GET_VERSION, ver);
}


/* Control */
int hci_get_devs(int *devs);
int hci_add_pin(BD_ADDR *bda, int Length, __u8 *Code);
int hci_remove_pin(BD_ADDR *bda);
int hci_add_key(BD_ADDR *bda, __u8 key_type, __u8 *key);
int hci_remove_key(BD_ADDR *bda);

int hci_set_dbmask(__u32 dbmask);
int hci_get_dbmask(__u32 *dbmask);

int hci_open_uart(char *name, int type, int proto, int speed, int flags);
int hci_close_uart(char *name);
int hci_setup_uart(char *name, int proto, int speed, int flags);

int hci_set_audio(int fd, int mode, __u16 setting);
int hci_get_conn(int fd, BD_ADDR *bda);



/* socket stuff */

static inline int hci_getmtu(int fd)
{
	int		err, mtu;
	socklen_t	len;

	err = getsockopt(fd, SOL_AFFIX, BTSO_MTU, &mtu, &len);
	if (err < 0)
		return err;

	return mtu;
}

/* L2CAP */
static inline int l2cap_getmtu(int fd)
{
	return hci_getmtu(fd);
}

static inline int l2cap_setmtu(int fd, int mtu)
{
	return setsockopt(fd, SOL_AFFIX, BTSO_MTU, &mtu, sizeof(mtu));
}

static inline int l2cap_flush(int fd)
{
	return ioctl(fd, SIOCL2CAP_FLUSH, 0);
}

int l2cap_sendping(int fd, char *data, int size, int type);
int l2cap_ping(int fd, char *data, int size);

static inline int l2cap_singleping(int fd, char *data, int size)
{
	return l2cap_sendping(fd, data, size, L2CAP_SINGLEPING);
}


/* RFCOMM stuff */
int rfcomm_get_ports(struct rfcomm_port *pi, int size);

static inline int rfcomm_open_tty(int fd, int line)
{
	int	err;

	err = ioctl(fd, SIOCRFCOMM_BINDPORT, &line);
	if (err < 0)
		return err;
	return line;
}

static inline int rfcomm_close_tty(int line)
{
	int	fd;

	fd = socket(PF_AFFIX, SOCK_STREAM, BTPROTO_RFCOMM);
	if (fd < 0)
		return fd;
	return ioctl(fd, SIOCRFCOMM_DISCONNECTPORT, &line);
}

static inline int rfcomm_set_type(int fd, int type)
{
	return ioctl(fd, SIOCRFCOMM_SETTYPE, &type);
}


int affix_init_pan(char *name, int mode);

__END_DECLS

#endif
