/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001,2002 Nokia Corporation
   Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   Original Author: Guruprasad Krishnamurthy <kgprasad@hotmail.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   */

/*
   $Id: cstate.h,v 1.4 2002/10/28 11:58:22 kds Exp $

   Fixes:
   		Dmitry Kasatkin		: cleanup, fixes
*/


#ifndef SDP_CSTATE_H
#define SDP_CSTATE_H

#include <stdint.h>

#include <affix/sdp.h>

/* Continuation state on client side */
typedef struct CState {
	uint8_t		length;
	uint8_t		data[16];
} __PACK__ CState;

int copyContinuationState(char *pdata, CState *pCState);

/* Continuation state on server side */
typedef struct {
	long timestamp;
	union {
		uint16_t maxBytesSent;
		uint16_t lastIndexSent;
	} cStateValue;
} ContinuationState;

#define ContinuationStateSize (sizeof(uint8_t) + sizeof(ContinuationState))

typedef struct {
	int		fd;		/* owner */
	long 		timestamp;	/* id */
	PDUForm 	responseBuffer;	/* actuall data */
} ContinuationStateBuffer;

ContinuationState *getContinuationState(char *buffer);

long createContinuationStateBuffer(int fd, PDUForm *response);
PDUForm *getCachedResponse(int fd, ContinuationState *pCState);
void removeCachedResponse(int fd, ContinuationState *pCState);

void printContinuationState(ContinuationState *pCState);

int setContinuationStatePDU(PDUForm *pduForm, ContinuationState *pCState);
int setContinuationState(char *pdata, ContinuationState *pCState);

void cleanupContinuationStateBuffer(void);

void initContinuationStateCache(void);

#endif 
