/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001,2002 Nokia Corporation
   Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   Original Author: Guruprasad Krishnamurthy <kgprasad@hotmail.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
   $Id: utils.h,v 1.9 2002/11/11 10:55:04 kds Exp $

   Miscellaneous implementation functions for use by the
   top level APIs

   Fixes:
   		Dmitry Kasatkin		- fixes
*/
	 
#ifndef SDP_UTILS_INT_H
#define SDP_UTILS_INT_H

#include <sys/ioctl.h>
#include <sys/socket.h>
#include <unistd.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <affix/bluetooth.h>
#include <affix/btcore.h>
#include <affix/sdp.h>


extern OperationMode	sdp_mode;


uint16_t generateTransactionId(void);
long getTimeInSeconds(void);
int sdp_init(OperationMode mode);
void sdp_shutdown(void);
void closeConnection(int socketFd);
unsigned char checkLocalConnection(SDPServerHandle handle);
int getMTUFromFd(int socketFd);

char *extractSequenceType(
  char *dst, 
  DataTypeDescriptor *pdataType,
  int *seqLength,
  int *bytesScanned
);

int generateAttrIDSequencePduForm(
  char *pdata,
  slist_t *pSeq,
  DataTypeDescriptor dataType
);

int generateSearchSequencePduForm(
  char *pdata,
  slist_t *pSeq
);

int extractServiceRecordHandles(
  char *pdata,
  slist_t **pSeq,
  int handleCount,
  int *bytesScanned
);

int extractDataElementSequence(
  char *pdata,
  slist_t **pSeq,
  DataTypeDescriptor *pDataType,
  DataTypeDescriptor expectedType
);


int sendRequestWaitForResponse(
  SDPServerHandle srvHandle,
  char *requestBuffer,
  char *responseBuffer,
  int requestSize,
  int *responseSize
);

#endif //SDP_UTILS_INT_H

